# Person_Localization_Client.py

"""Person_Localization_Client
"""

import threading
import xmlrpc.client

from pyrois import RoIS_Common, RoIS_HRI


class Command(RoIS_Common.Command):
    """Command
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def start(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.start())
        return status

    def stop(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.stop())
        return status

    def suspend(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.suspend())
        return status

    def resume(self):
        status = RoIS_HRI.ReturnCode_t(self._proxy.resume())
        return status

    def set_parameter(self, detection_threshold, minimum_interval):
        s = self._proxy.set_parameter(detection_threshold, minimum_interval)
        status = RoIS_HRI.ReturnCode_t(s)
        return status
    

class Query(RoIS_Common.Query):
    """Query
    """
    def __init__(self, uri):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)

    def component_status(self):
        (status, c_status) = self._proxy.component_status()
        return (RoIS_HRI.ReturnCode_t(status), RoIS_Common.Component_Status(c_status))
    
    def get_parameter(self):
        (status, detection_threshold, minimum_interval) = self._proxy.get_parameter()
        return (RoIS_HRI.ReturnCode_t(status), detection_threshold, minimum_interval)


class Event(RoIS_Common.Event):
    """Event
    """
    def __init__(self, uri):
        self._uri = uri
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
                
    def start_th(self):
        self.th = threading.Thread(target=self.event_loop)
        self.th.start()

    def event_loop(self):
        """event_loop
        """
        while True:
            try:
                self.poll_event()
            except ConnectionRefusedError:
                break

    def poll_event(self):
        """poll_event
        """
        msg = self._e_proxy.poll_event()
        (params, methodname) = xmlrpc.client.loads(msg)
        print(params,methodname)
        if methodname == 'person_localized':
            self.person_localized(params[0], params[1], params[2])

    def person_localized(self, timestamp, person_ref, position_data):
        self.events.append((timestamp,person_ref,position_data))
        print("person_localized",timestamp, person_ref, position_data)


class Person_Localization_Client(Command, Query, Event):
    """Person_localizaion_Client
    """
    def __init__(self, uri, engine_event_queue):
        self._uri = uri
        self._proxy = xmlrpc.client.ServerProxy(self._uri)
        self._e_proxy = xmlrpc.client.ServerProxy(self._uri)
        self.events = []
        self.engine_event_queue = engine_event_queue
        self.start_th()

    def poll_event(self):
        """poll_event
        """
        msg = self._e_proxy.poll_event()
        self.engine_event_queue.put(msg)
